"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useShift = exports.useIsSeriesVisible = exports.usePointPosition = exports.useDirectedEllipsePosition = exports.useEllipsePosition = exports.useRectanglePosition = exports.usePosition = void 0;
const ml_distance_euclidean_1 = require("ml-distance-euclidean");
const legendContext_1 = require("./contexts/legendContext");
const plotContext_1 = require("./contexts/plotContext");
const utils_1 = require("./utils");
function usePosition(config) {
    const { axisContext, plotWidth, plotHeight } = (0, plotContext_1.usePlotContext)();
    const [xScale, yScale] = (0, utils_1.validateAxis)(axisContext, 'x', 'y');
    const { x, y } = config;
    return {
        x: convertValue(x, plotWidth, xScale),
        y: convertValue(y, plotHeight, yScale),
    };
}
exports.usePosition = usePosition;
function useRectanglePosition(config) {
    const { axisContext, plotWidth, plotHeight } = (0, plotContext_1.usePlotContext)();
    const [xScale, yScale] = (0, utils_1.validateAxis)(axisContext, 'x', 'y');
    const { x1, y1, x2, y2 } = config;
    return {
        x: convertMinValue(x1, x2, plotWidth, xScale),
        y: convertMinValue(y1, y2, plotHeight, yScale),
        width: convertDimensions(x1, x2, plotWidth, xScale),
        height: convertDimensions(y1, y2, plotHeight, yScale),
    };
}
exports.useRectanglePosition = useRectanglePosition;
function useEllipsePosition(props) {
    const { axisContext, plotWidth, plotHeight } = (0, plotContext_1.usePlotContext)();
    const [xScale, yScale] = (0, utils_1.validateAxis)(axisContext, 'x', 'y');
    const { cx, cy, rx, ry } = props;
    return {
        cx: convertValue(cx, plotWidth, xScale),
        cy: convertValue(cy, plotHeight, yScale),
        rx: convertValueAbs(rx, plotWidth, xScale),
        ry: convertValueAbs(ry, plotHeight, yScale),
    };
}
exports.useEllipsePosition = useEllipsePosition;
function useDirectedEllipsePosition(props) {
    const { axisContext, plotWidth, plotHeight } = (0, plotContext_1.usePlotContext)();
    const [xScale, yScale] = (0, utils_1.validateAxis)(axisContext, 'x', 'y');
    const { x1: oldX1, y1: oldY1, x2: oldX2, y2: oldY2, width } = props;
    const { x1, y1, x2, y2 } = {
        x1: convertValue(oldX1, plotWidth, xScale),
        x2: convertValue(oldX2, plotWidth, xScale),
        y1: convertValue(oldY1, plotWidth, yScale),
        y2: convertValue(oldY2, plotWidth, yScale),
    };
    const radsToDegs = (rad) => (rad * 180) / Math.PI;
    const { cx, cy } = {
        cx: (x1 + x2) / 2,
        cy: (y1 + y2) / 2,
    };
    const rotation = (y1 > y2 ? -1 : 1) *
        (x1 > x2 ? -1 : 1) *
        Math.asin((0, ml_distance_euclidean_1.euclidean)([x1, y1], [x1, cy]) / (0, ml_distance_euclidean_1.euclidean)([x1, y1], [cx, cy]));
    const { widthX, widthY } = {
        widthX: (Math.sin(rotation) * convertValueAbs(width, plotHeight, xScale)) / 2,
        widthY: (Math.cos(rotation) * convertValueAbs(width, plotHeight, yScale)) / 2,
    };
    return {
        cx,
        cy,
        rx: (0, ml_distance_euclidean_1.euclidean)([x1, y1], [x2, y2]) / 2,
        ry: (0, ml_distance_euclidean_1.euclidean)([0, 0], [widthX, widthY]),
        rotation: radsToDegs(rotation),
    };
}
exports.useDirectedEllipsePosition = useDirectedEllipsePosition;
function convertString(value, total) {
    return value.endsWith('%')
        ? (Number(value.slice(0, -1)) * total) / 100
        : Number(value);
}
function convertValue(value, total, scale) {
    if (scale === undefined)
        return 0;
    return typeof value === 'number' ? scale(value) : convertString(value, total);
}
function convertMinValue(value1, value2, total, scale) {
    if (scale === undefined)
        return 0;
    return Math.min(typeof value2 === 'number' ? scale(value2) : convertString(value2, total), typeof value1 === 'number' ? scale(value1) : convertString(value1, total));
}
function convertValueAbs(value, total, scale) {
    if (scale === undefined)
        return 0;
    return typeof value === 'number'
        ? Math.abs(scale(0) - scale(value))
        : Math.abs(convertString(value, total));
}
function convertToPx(value, total, scale) {
    if (scale === undefined)
        return 0;
    return typeof value === 'number'
        ? scale(value) - scale(0)
        : convertString(value, total);
}
function convertDimensions(value1, value2, total, scale) {
    if (scale === undefined)
        return 0;
    return Math.abs((typeof value2 === 'number'
        ? scale(value2)
        : convertString(value2, total)) -
        (typeof value1 === 'number'
            ? scale(value1)
            : convertString(value1, total)));
}
function usePointPosition(config) {
    const { axisContext, plotWidth, plotHeight } = (0, plotContext_1.usePlotContext)();
    const [xScale, yScale] = (0, utils_1.validateAxis)(axisContext, 'x', 'y');
    const points = config;
    return points
        .map((point) => `${convertValue(point.x, plotWidth, xScale)},${convertValue(point.y, plotHeight, yScale)}`)
        .join(' ');
}
exports.usePointPosition = usePointPosition;
function useIsSeriesVisible(id) {
    const [legendState] = (0, legendContext_1.useLegend)();
    const value = legendState.labels.find((label) => label.id === id);
    return value ? value.isVisible : true;
}
exports.useIsSeriesVisible = useIsSeriesVisible;
function useShift(options) {
    const { axisContext, plotWidth, plotHeight } = (0, plotContext_1.usePlotContext)();
    const { xAxis, yAxis, xShift, yShift } = options;
    const [xScale, yScale] = (0, utils_1.validateAxis)(axisContext, xAxis, yAxis);
    return {
        xShift: convertToPx(xShift, plotWidth, xScale),
        yShift: convertToPx(yShift, plotHeight, yScale),
    };
}
exports.useShift = useShift;
